/*!
 * type-reverse <https://github.com/whizkydee/type-reverse>
 *
 * Copyright (c) 2017-present, Olaolu Olawuyi.
 * Released under the MIT License.
 */

'use strict';

/**
 * 🦄 Lightweight reverse utility around strings, arrays, numbers and more.
 *
 * @name reverse
 * @alias inverse
 * @param {String|Number|Array|Set|NodeList} `input`
 * @param {?Object} `options`
 * @param {?Function} `callback`
 * @return {*}
 * @api public
 */

Object.defineProperty(exports, "__esModule", {
  value: !0
});

var _util = require('./util');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } return Array.from(arr); }

function reverse(input) {
  var options = 1 < arguments.length && arguments[1] !== undefined ? arguments[1] : {},
      callback = arguments[2];
  options = !!options ? options : new Object();


  var newReversedArray = [].concat(_toConsumableArray(input)).reverse(),
      preserveZeros = options.preserveZeros,
      minusRE = /^-/;

  if (input && !(0, _util.supported)(input)) throw new TypeError("Failed to apply 'reverse': " + (0, _util.kindof)(input) + 's are not supported');

  options.invert = options.invert || 'index', options.preserveZeros = preserveZeros || !1, callback = callback || function (_, v) {
    return v;
  };


  var result = void 0;
  switch ((0, _util.kindof)(input)) {
    case 'string':
      switch (options.invert) {
        case 'index':
          result = newReversedArray.join('');
          break;
        case 'word':
          result = input.split(' ').reverse().join(' ');

      }
      break;

    case 'number':
      // convert the number to string then replace the minus(-) symbol with nothing
      var positiveNumberAsString = String(input).replace(minusRE, ''),
          bigInt = /e/i.test(positiveNumberAsString),
          containsTrailingZeros = /0+$/.test(positiveNumberAsString),
          noNeedToStringify = containsTrailingZeros && !preserveZeros || !containsTrailingZeros && preserveZeros || !containsTrailingZeros && !preserveZeros;

      // there's no need to stringify if...
      // 1. the input contains trailing zeros, but the preserveZeros option is disabled
      // 2. the input doesn't contain trailing zeros, but the preserveZeros option is enabled
      // 3. the input doesn't contain trailing zeros, and the preserveZeros option is disabled

      if (bigInt) throw new TypeError('Oops. That number is too large. See https://github.com/whizkydee/type-reverse/blob/master/readme.md#limits for more info.');

      switch (options.invert) {
        case 'sign':
          result = noNeedToStringify ? minusRE.test(input) ? Number(+positiveNumberAsString) : Number(-positiveNumberAsString) : minusRE.test(input) ? positiveNumberAsString : '-' + positiveNumberAsString;

          break;
        case 'index':
          result = noNeedToStringify ? minusRE.test(input) ? reverse(positiveNumberAsString, null, function (_, x) {
            return Number(-x);
          }) : reverse(positiveNumberAsString, null, function (_, x) {
            return Number(x);
          }) : minusRE.test(input) ? reverse(positiveNumberAsString, null, function (_, x) {
            return '-' + x;
          }) : reverse(positiveNumberAsString, null, function (_, x) {
            return x;
          });

      }
      break;

    case 'set':
      result = new Set(newReversedArray);
      break;
    case 'array':case 'nodelist':
      result = newReversedArray;
      break;

    default:
      result = reverse(input);

  }

  if ('function' === typeof callback) return callback.call(this, input, result);
  if (callback && 'function' !== typeof callback) throw new TypeError("Failed to apply 'reverse': Expected function as third argument, got " + (0, _util.kindof)(callback) + '.');

  return result;
}

exports.default = reverse;
module.exports = reverse;